% This script supports the creation of a custom reference design (including
% the XADC) by providing some useful code

% Requirements: Vivado 2014.4
%               MathWorks hardware support package "Xilinx Zynq-7000"
% File references (in same folder): createProject.tcl
%                                   start_vivado.tcl

%% change the work path to the file location
msgbox('Make sure that you are in the right folder!');

%% 1: create Vivado Project:
% the following code creates a vivado project using an existing template
% from the hardware support package (system_top.tcl)

% this line registers Vivado as an external toolchain
hdlsetuptoolpath('ToolName', 'Xilinx Vivado', ...
    'ToolPath', 'C:\Xilinx\Vivado\2014.4\bin\vivado.bat');
% please ensure that the Vivado path is correct

projectName = 'system_top';
toolboxPath = which('hdlcoder_board_customization','-all');
idx = ~cellfun(@isempty,(strfind(toolboxPath, 'zynq7000')));
toolboxPath = fileparts(toolboxPath{idx});

% creates template project in new folder vivado_project
system(['vivado -mode tcl',...
    ' -source createProject.tcl',...     % creates a new zedboard project
    ' -source ', toolboxPath, '\+ZedBoard\+vivado_base_2014_4\system_top.tcl',... % loads template
    ' -source start_vivado.tcl &'])  

%% 2: customize hardware design
% design hardware ip cores upon existing reference design
% connect all ip cores except those to Simulink PL Canvas
% tcl-command to save just created new block design as tcl-script 
% (system_top.tcl) in the current folder:
% set path [pwd]
% write_bd_tcl $path/system_top.tcl

%% 3: write list of interfaces
% open excel sheet interface.xlsx
winopen('interface.xlsx');
edit('XADC_vpvn_vaux0_vaux8.xdc');

%% 4: copy template for custom reference design
% define a path where to put the reference design
% define a boardName for the custom reference design
boardDefPath = fullfile(pwd, 'MyZynqBoards');
boardFolderName = '+XADCDemo';    % needs to be a package folder (+)
%
% copy template hardware reference design to boardDefPath
mkdir(fullfile(boardDefPath,boardFolderName,'+vivado_base_2014_4','ipcore'));

% Copy the Board plugin registration file
copyfile(fullfile(toolboxPath,'hdlcoder_board_customization.m'),...
    fullfile(boardDefPath,'hdlcoder_board_customization.m'));

% Copy the Board definition file
copyfile(fullfile(toolboxPath,'+ZedBoard','plugin_board.m'),...
    fullfile(boardDefPath,boardFolderName,'plugin_board.m'));

% Copy the Reference design plugin registration file
copyfile(fullfile(toolboxPath,'+ZedBoard','hdlcoder_ref_design_customization.m'),...
    fullfile(boardDefPath,boardFolderName,'hdlcoder_ref_design_customization.m'));

% Copy the Reference design definition file
copyfile(fullfile(toolboxPath,'+ZedBoard','+vivado_base_2014_4','plugin_rd.m'),...
    fullfile(boardDefPath,boardFolderName,'+vivado_base_2014_4','plugin_rd.m'));

%% 5: copy created reference design and custom ip cores
% copy system_top.tcl file to
% copyfile(fullfile('savepath','system_top.tcl'),...
% fullfile(boardDefPath,boardName,'+vivado_base_2014_4','system_top.tcl'));
% copy all constraints file in folder: 
% copyfile(fullfile('savepath','contraint.xdc'),...
% fullfile(boardDefPath,boardName,'+vivado_base_2014_4','constraint.xdc'));

% pack custom ip cores in zip container
% zip(fullfile(boardDefPath,boardName,'+vivado_base_2014_4','ipcore',...
% 'custom_ipcore.zip'),fullfile('custom_ipcore'));

%% 6: edit board plugin registration file
% * Open the board plugin registration file
% 'hdlcoder_board_customization.m'
% * Delete all board plugins except the ZedBoard plugin
% * Rename ZedBoard.plugin_board to <boardFolderName>.plugin_board 
% (where boardFolderName is the variable defined before, without +
% character)
% * Save the changes
edit(fullfile(boardDefPath, 'hdlcoder_board_customization.m'));

%% 7: edit board definition file
% * Open the board definition file
% * Change the board name to *XADC Demo* (this name appears in the HDL
% Workflow Advisor at task 1.1 in the Target Platform list)
% * Delete 'Xilinx ISE' as supported tool
% * Add additional external I/O interface as defined in the file 
% 'interface.xslx'
% * Save the changes
edit(fullfile(boardDefPath, boardFolderName,'plugin_board.m'));

%% 8: edit reference design plugin registration file
% * Open the reference design plugin registration file
% * Delete all reference design plugins except that for Vivado 2014.4 Base
% * Rename the reference design plugin ZedBoard.vivado... to XADCDemo.vivado...
% * Change the board name to *XADC Demo*
% * Save the changes
edit(fullfile(boardDefPath,boardFolderName,'hdlcoder_ref_design_customization.m'));

%% 9: edit reference design definition file
% * Open the board definition file
% * Change the board name to *XADC Demo*
% * Make sure the right Synthesis Tool and Version is set, and
% that the name of the reference design file is correct.
% * Add custom constraint file definition, therefore insert 
% the following command below the command hRD.addCustomVivadoDesign (line 21)
% hRD.CustomConstraints = {'XADC_vpvn_vaux0_vaux8.xdc'};
% * Add custom IP cores to the reference design, therefore insert 
% the following command below the just inserted command
% hRD.CustomFiles = {'ipcore'};
% * Check the clock and AXI4 interface definition (when you created the
% reference design with the original tcl-script |system_top.tcl| then it
% should match. Otherwise you need to change it.)
% * Add internal interfaces to custom ip cores
% * Save the changes
edit(fullfile(boardDefPath,boardFolderName,'+vivado_base_2014_4','plugin_rd.m'));

%% 10: add board definition folder to Matlab search path
addpath(boardDefPath);  % add this path in your startup script to make it 
% available
% rmpath(boardDefPath); % ro remove path
% savepath;   % to save path persistent



